


# ---- Genetic contribution of vgll3 and six6 to among-year variation in sea age ----------- 

# In this script we calculate: 

# Variance in sea age among years in females and males 
# Average additive and dominance effect of vgll3 and six6 on sea age in females and males
# Correlation between females and males in mean sea age 
# The contribution of additive and dominance effects of vgll3 and six6 to the variance in mean sea age among years 
# Fig 1 


# ---- Source the script that prepares the dataset and runs models ---------------------------------------------------------------------------------------
source("00-run-prep-scripts.R")


# ---- How much does sea age vary among years?  --------------------------------------------------------------------------------------------

# Females 

# range:
min(coef(summary(modF))[1:6, "Estimate"])      
max(coef(summary(modF))[1:6, "Estimate"])      

# variance:
varF <- var(coef(summary(modF))[1:6, "Estimate"])                              # among-year variance without controlling for genetic effects
varF_error_corrected  <- varF - mean(coef(summary(modF))[1:6, "Std. Error"]^2) # error-corrected variance without controlling for genetic effects
varF_percent_uncertainty <- (varF - varF_error_corrected)/varF*100             # 34% of variance explained by uncertainty 
SDF_error_corrected <- sqrt(varF_error_corrected)                              # error-corrected SD among years 

# Males

# range:
min(coef(summary(modM))[1:6, "Estimate"])      
max(coef(summary(modM))[1:6, "Estimate"])      

varM <- var(coef(summary(modM))[1:6, "Estimate"])                              # among-year variance without controlling for genetic effects
varM_error_corrected  <- varM - mean(coef(summary(modM))[1:6, "Std. Error"]^2) # error-corrected variance without controlling for genetic effects
varM_percent_uncertainty <- (varM - varM_error_corrected)/varM*100             # 15.9% of variance explained by uncertainty 
SDM_error_corrected <- sqrt(varM_error_corrected)                              # error-corrected SD among years 


# ---- Variance among years explained by additive and dominance effects of vgll3 and six6 ----------------------------------------------------

# Females

varF_vgll3_six6 <- var(coef(summary(modF_vgll3_six6))[1:6, "Estimate"])        # among-year variance when controlling for genetic effects
varF_percent_vgll3_six6 <- (varF - varF_vgll3_six6)/varF*100                   # among-year variance decreases by 28% when controlling for genetic effects 
100 - (varF_percent_uncertainty + varF_percent_vgll3_six6)                     # 38% of the variance not due to genetic effects or uncertainty                            


# Males 
varM_vgll3_six6 <- var(coef(summary(modM_vgll3_six6))[1:6, "Estimate"])        # among-year variance when controlling for genetic effects
varM_percent_vgll3_six6 <- (varM - varM_vgll3_six6)/varM*100                   # among-year variance decreases by 15.8 % when controlling for genetic effects 
100 - (varM_percent_uncertainty + varM_percent_vgll3_six6)                     # 68% of the variance not due to genetic effects or uncertainty   


sqrt(varF_error_corrected)
sqrt(varM_error_corrected)

# ---- Correlation females and males ---------------------------------------------------------------------------------------------------------


cor.test(coef(modF), coef(modM))                                               # the sexes are positively correlated in mean sea age by 0.84


# ---- Average additive and dominance effects ------------------------------------------------------------------------------------------------

# estimate_SE <-function(estimate, SE){
#   paste0(round(estimate, 2), sep = " ", "B1", sep = " ", round(SE, 2))
# }

# vgll3 - females
add_f_vgll3 <- coef(summary(modF_vgll3_six6))["a_vgll3", "Estimate"] 
dom_f_vgll3 <- coef(summary(modF_vgll3_six6))["d_vgll3", "Estimate"] 
add_f_vgll3*2 # diff EE LL 
add_f_vgll3*1 + dom_f_vgll3*1 # diff EE EL
add_f_vgll3*2 - (add_f_vgll3*1 + dom_f_vgll3*1) # diff EL and LL 

addse_f_vgll3 <- estimate_SE(add_f_vgll3, coef(summary(modF_vgll3_six6))["a_vgll3","Std. Error"])
domse_f_vgll3 <- estimate_SE(dom_f_vgll3, coef(summary(modF_vgll3_six6))["d_vgll3","Std. Error"])

# vgll3 - males
add_m_vgll3 <- coef(summary(modM_vgll3_six6))["a_vgll3", "Estimate"] 
dom_m_vgll3 <- coef(summary(modM_vgll3_six6))["d_vgll3", "Estimate"] 

add_m_vgll3*2 # diff EE LL 
add_m_vgll3*1 + dom_m_vgll3*1 # diff EE EL
add_m_vgll3*1 - (add_m_vgll3*1 + dom_m_vgll3*1) # diff EL and LL 

addse_m_vgll3 <- estimate_SE(add_m_vgll3, coef(summary(modM_vgll3_six6))["a_vgll3","Std. Error"])
domse_m_vgll3 <- estimate_SE(dom_m_vgll3, coef(summary(modM_vgll3_six6))["d_vgll3","Std. Error"])


# six6 - females 
addse_f_six6 <- estimate_SE(coef(summary(modF_vgll3_six6))["a_six6", "Estimate"], coef(summary(modF_vgll3_six6))["a_six6","Std. Error"])
domse_f_six6 <- estimate_SE(coef(summary(modF_vgll3_six6))["d_six6", "Estimate"] , coef(summary(modF_vgll3_six6))["d_six6","Std. Error"])

# six6 - males
addse_m_six6 <- estimate_SE(coef(summary(modM_vgll3_six6))["a_six6", "Estimate"], coef(summary(modM_vgll3_six6))["a_six6","Std. Error"])
domse_m_six6 <- estimate_SE(coef(summary(modM_vgll3_six6))["d_six6", "Estimate"] , coef(summary(modM_vgll3_six6))["d_six6","Std. Error"])





#---- Figure 1A: seaage over time in females and males with and without controlling for vgll3 and six6 ----------------------------------------- 
  
  # predictions for each year without controlling for vgll3 and six6
  p_wo <- rbind(
    tidy(modF) %>% 
      mutate(year_smolt = min(df$year_smolt):max(df$year_smolt),
             Sex = "Female",
             seaage = estimate),
    tidy(modM) %>% 
      mutate(year_smolt = min(dm$year_smolt):max(dm$year_smolt),
             Sex = "Male",
             seaage = estimate)
  )
p_wo$Model <- "Not controlling for genetic effects"

# with controlling for vgll3 and six6 - predictions for each year for global average values of a_vgll3, d_vgll3, a_six6 and d_six6: 
p_w <- rbind(tidy(modF_vgll3_six6) %>% 
               filter(!term == "a_vgll3" & !term == "d_vgll3" & !term == "a_six6" & !term == "d_six6") %>% 
               mutate(Sex = "Female",
                      seaage = estimate + coef(summary(modF_vgll3_six6))["a_vgll3","Estimate"]*mean(df$a_vgll3) + coef(summary(modF_vgll3_six6))["d_vgll3","Estimate"]*mean(df$d_vgll3)
                      + coef(summary(modF_vgll3_six6))["a_six6","Estimate"]*mean(df$a_six6) + coef(summary(modF_vgll3_six6))["d_six6","Estimate"]*mean(df$d_six6), # year estimate + a and d effect of mean a and d
                      year_smolt = min(df$year_smolt):max(df$year_smolt)),
             tidy(modM_vgll3_six6) %>%
               filter(!term == "a_vgll3" & !term == "d_vgll3"& !term == "a_six6" & !term == "d_six6") %>% 
               mutate(Sex = "Male",
                      seaage = estimate + coef(summary(modM_vgll3_six6))["a_vgll3","Estimate"]*mean(dm$a_vgll3) + coef(summary(modM_vgll3_six6))["d_vgll3","Estimate"]*mean(dm$d_vgll3)
                      + coef(summary(modM_vgll3_six6))["a_six6","Estimate"]*mean(dm$a_six6) + coef(summary(modM_vgll3_six6))["d_six6","Estimate"]*mean(dm$d_six6),
                      year_smolt = min(dm$year_smolt):max(dm$year_smolt)))

p_w$Model <- "Controlling for genetic effects"


p_w_wo <- rbind(p_w, p_wo)


# for displaying individual data of females and males - sea age 3 or older collapsed into one category in the fig:
sex_seaage <- d %>% dplyr::select(seaAge_firstspawn, sex, year_smolt, vgll3, six6)
sex_seaage$seaAge_firstspawn_123plus <-  with(sex_seaage, ifelse(seaAge_firstspawn > 3, 3, seaAge_firstspawn))
sex_seaage$Sex  <- sex_seaage$sex



fig_seaage_over_time <- 
  p_w_wo %>% ggplot(aes(x = year_smolt, y = seaage, col = Sex))+
  geom_line(aes(linetype = Model))+
  geom_text(x = 2017, y = 3.1, label = expression(sigma == "0.10"), col = "#E69F00")+
  geom_text(x = 2017, y = 2.9, label = expression(sigma == "0.16"), col = "#000000")+
  geom_point(data = subset(p_w_wo, Model == "Not controlling for genetic effects"), size = 1)+
  geom_errorbar(data = subset(p_w_wo, Model == "Not controlling for genetic effects"), aes(ymin = seaage - std.error, ymax = seaage + std.error), width = 0.1)+
  geom_jitter(data = sex_seaage, aes(x = year_smolt, y = seaAge_firstspawn_123plus, col = Sex), alpha = 0.1, width = 0.3, height = 0.2, size = 0.5)+
  scale_linetype_manual(values = c("dashed", "solid"))+
  scale_y_continuous("Sea age (yr)", breaks = seq(1,3, 1))+
  scale_x_continuous("Year of outmigration", breaks = seq(2013, 2018, 1))+
  scale_color_manual(values = c("#E69F00", "#000000"))
fig_seaage_over_time


# ---- Figure 1B: effect of vgll3 genotype on sea age in females and males ---------------------------------------------------------------------------

EEf <- ggpredict(modF_vgll3_six6, type = "fe", terms = c("a_vgll3[0]", "d_vgll3[0]", "seaAge_correction[0]"))
ELf <- ggpredict(modF_vgll3_six6, type = "fe", terms = c("a_vgll3[1]", "d_vgll3[1]",  "seaAge_correction[0]"))
LLf <- ggpredict(modF_vgll3_six6, type = "fe", terms = c("a_vgll3[2]", "d_vgll3[0]",  "seaAge_correction[0]"))
pf <- rbind(EEf, ELf, LLf)
pf$Sex <- "Female"

EEm <- ggpredict(modM_vgll3_six6, type = "fe", terms = c("a_vgll3[0]", "d_vgll3[0]",  "seaAge_correction[0]"))
ELm <- ggpredict(modM_vgll3_six6, type = "fe", terms = c("a_vgll3[1]", "d_vgll3[1]",  "seaAge_correction[0]"))
LLm <- ggpredict(modM_vgll3_six6, type = "fe", terms = c("a_vgll3[2]", "d_vgll3[0]", "seaAge_correction[0]"))
pm <- rbind(EEm, ELm, LLm)
pm$Sex <- "Male"
p <- rbind(pf, pm)


fig_vgll3 <- p  %>% ggplot(aes(x = x, y = predicted, col = Sex))+
  geom_point(size = 1)+
  geom_line()+
  geom_jitter(data = sex_seaage, aes(x = vgll3, y = seaAge_firstspawn_123plus, col = Sex), alpha = 0.1, width = 0.3, height = 0.2, size = 0.5)+
  scale_color_manual(values = c("#E69F00", "#000000"))+
  geom_errorbar(aes(ymin = predicted - std.error, ymax = predicted + std.error), width = 0.1)+
  scale_y_continuous("Sea age (yr)", breaks = seq(1,3, 1))+
  scale_x_continuous(expression(paste(italic("vgll3"), " genotype")), breaks = seq(0, 2, 1), labels = c("EE", "EL", "LL"))
fig_vgll3


# ---- Figure 1C: effect of six6 genotype on sea age in females and males ---------------------------------------------------------------------------

EEf <- ggpredict(modF_vgll3_six6, type = "fe", terms = c("a_six6[0]", "d_six6[0]", "seaAge_correction[0]"))
ELf <- ggpredict(modF_vgll3_six6, type = "fe", terms = c("a_six6[1]", "d_six6[1]",  "seaAge_correction[0]"))
LLf <- ggpredict(modF_vgll3_six6, type = "fe", terms = c("a_six6[2]", "d_six6[0]",  "seaAge_correction[0]"))
pf <- rbind(EEf, ELf, LLf)
pf$Sex <- "Female"

EEm <- ggpredict(modM_vgll3_six6, type = "fe", terms = c("a_six6[0]", "d_six6[0]",  "seaAge_correction[0]"))
ELm <- ggpredict(modM_vgll3_six6, type = "fe", terms = c("a_six6[1]", "d_six6[1]",  "seaAge_correction[0]"))
LLm <- ggpredict(modM_vgll3_six6, type = "fe", terms = c("a_six6[2]", "d_six6[0]", "seaAge_correction[0]"))
pm <- rbind(EEm, ELm, LLm)
pm$Sex <- "Male"
p <- rbind(pf, pm)

# plot for 2015 because this is a year with an average sea age 
fig_six6 <- p  %>% ggplot(aes(x = x, y = predicted, col = Sex))+
  geom_point(size = 1)+
  geom_line()+
  geom_jitter(data = sex_seaage, aes(x = six6, y = seaAge_firstspawn_123plus, col = Sex), alpha = 0.1, width = 0.3, height = 0.2, size = 0.5)+
  scale_color_manual(values = c("#E69F00", "#000000"))+
  geom_errorbar(aes(ymin = predicted - std.error, ymax = predicted + std.error), width = 0.1)+
  scale_y_continuous("Sea age (yr)", breaks = seq(1,3, 1))+
  scale_x_continuous(expression(paste(italic("six6"), " genotype")), breaks = seq(0, 2, 1), labels = c("EE", "EL", "LL"))
fig_six6

# ---- FIGURE 1 -------------------------------------------------------------------------------------------------------------------------------------


Fig1 <- ggarrange(fig_seaage_over_time,
                  labels = "A",
                  ggarrange(fig_vgll3 + guides(col = "none"), fig_six6 + guides(col = "none"), labels = c("B", "C")), 
                  nrow = 2, 
                  widths = c(0.1, 1, 1))

Fig1


ggsave(Fig1, path = "figs/",  filename = "Figure_1.pdf", width = 16, height = 16, units = "cm")


