
# ---- Genotype-year interaction effects on sea age  ----------

# In this script we make: 

# predictions of mean sea age of each genotype and sex within each year 
# % of the variation among years and genotypes explained by G x Year interaction (Table 1)

# ---- Source the script that prepares the dataset and runs models ---------------------------------------------------------------------------------------
source("00-run-prep-scripts.R")


# ---- Predicted mean sea age for each six6 genotype in each year  ---------------------------------------------------------------------------------------------------

# Females

# set dominance of six6 to 0 for females in 2015, can't estimate the dominance effect because one genotype is lacking
newdataF_six6 <- as_tibble(cbind(year_smolt = rep(c(2013:2018), times = 3),
                            a_six6 = rep(c(0, 1, 2), each = 6),
                            d_six6 = c(rep(0, times = 6), 1, 1, 0, 1, 1, 1, rep(0, times = 6)),# set dominance to 0 in 2015 
                            a_vgll3 = rep(mean(df$a_vgll3), times = 18),
                            d_vgll3 = rep(mean(df$d_vgll3), each = 18),
                            seaAge_correction = rep(0, times = 18)))

newdataF_six6 <- newdataF_six6 %>% filter(!(year_smolt == 2015 & a_six6 == 0)) # lack data of six6 EE genotype in 2015 

predicted <- predict(modF_vgll3_six6_year, newdata = newdataF_six6, se.fit = TRUE)
df_predicted_six6 <- as_tibble(cbind(predicted = predicted$fit, se = predicted$se.fit,
                                year_smolt = newdataF_six6$year_smolt, 
                                a_six6 = newdataF_six6$a_six6)) %>% mutate(Sex = "Female")




# Males

newdataM_six6 <- as_tibble(cbind(year_smolt = rep(c(2013:2018), times = 3),
                            a_six6 = rep(c(0, 1, 2), each = 6),
                            d_six6 = rep(c(0, 1, 0),each = 6), 
                            a_vgll3 = rep(mean(dm$a_vgll3), times = 18),   
                            d_vgll3 = rep(mean(dm$d_vgll3), each = 18),
                            seaAge_correction = rep(0, times = 18)))
predicted <- predict(modM_vgll3_six6_year, newdata = newdataM_six6, se.fit = TRUE)

dm_predicted_six6 <- as_tibble(cbind(predicted = predicted$fit,
                                     se = predicted$se.fit,
                                     year_smolt = newdataM_six6$year_smolt, 
                                     a_six6 = newdataM_six6$a_six6)) %>% mutate(Sex = "Male")

dfm_predicted_six6 <- rbind(df_predicted_six6, dm_predicted_six6)
dfm_predicted_six6$Genotype <- with(dfm_predicted_six6, ifelse(a_six6 == 0, "EE", ifelse(a_six6 == 1, "EL", "LL")))



# ------------ Among-year standard deviation of each genotype, corrected for uncertainty ------------------------

dfm_predicted_six6 <- dfm_predicted_six6 %>% group_by(Sex, Genotype) %>% mutate(variance = var(predicted),
                                                                              meanSE2 = mean(se^2),
                                                                              SD_corrected = sqrt(variance - meanSE2))

# ---------------- Fig 3A  --------------------------------------------------------------------------------------

# Individual sea age data: pool individuals with sea age > 3 with sea age 3 in the plot: 
d$seaAge_firstspawn_123plus <-  with(d, ifelse(seaAge_firstspawn > 3, 3, seaAge_firstspawn))
six6_seaage <- d %>% dplyr::select(year_smolt, sex, six6, seaAge_firstspawn_123plus)
six6_seaage$Genotype <- with(six6_seaage, ifelse(six6 == 0, "EE", ifelse(six6 == 1, "EL", "LL")))
six6_seaage$Sex <- six6_seaage$sex


# Error-corrected standard deviation for each six6 genotype to include in plot:
sd_six6 <- dfm_predicted_six6 %>% dplyr::select(Sex, Genotype, SD_corrected) %>% slice(1) %>% mutate(SD = round2(SD_corrected))
sd_six6$year_smolt <- c(rep(2017, times = 6))
sd_six6$predicted <- rep(c(2.9, 3.1, 3.3), times = 2)

# expression with sigma and genotype as subscript: 
sd_six6$label[sd_six6$Genotype == "EE" & sd_six6$Sex == "Female"] <- as.character(expression(sigma[EE] == 0.24))
sd_six6$label[sd_six6$Genotype == "EL" & sd_six6$Sex == "Female"] <- as.character(expression(sigma[EL] == 0.07))
sd_six6$label[sd_six6$Genotype == "LL" & sd_six6$Sex == "Female"] <- as.character(expression(sigma[LL] == 0.08))

sd_six6$label[sd_six6$Genotype == "EE" & sd_six6$Sex == "Male"] <- as.character(expression(sigma[EE] == 0.21))
sd_six6$label[sd_six6$Genotype == "EL" & sd_six6$Sex == "Male"] <- as.character(expression(sigma[EL] == 0.09))
sd_six6$label[sd_six6$Genotype == "LL" & sd_six6$Sex == "Male"] <- as.character(expression(sigma[LL] == 0.15))


dfm_predicted_six6 %>% ggplot(aes(x = year_smolt, y = predicted, col = Genotype))+
  geom_line(position = position_dodge(width = 0.3))+
  facet_wrap(.~ Sex)+
  geom_point(position = position_dodge(width = 0.3))+
  geom_text(data = sd_six6, aes(label = label), parse = T, show.legend = FALSE)+
  geom_errorbar(aes(ymin = predicted - se, ymax = predicted + se), position = position_dodge(width = 0.3), width = 0.5)+
  geom_jitter(data = six6_seaage, aes(x = year_smolt, y = seaAge_firstspawn_123plus, col = Genotype), alpha = 0.1, width = 0.3, height = 0.2, size = 1)+
  scale_y_continuous("Sea age (yr)", breaks = seq(1,3, 1))+
  scale_x_continuous("Year of outmigration", breaks = seq(2013, 2018, 1))+
  scale_color_manual(values = c("#CC79A7", "#009E73","#D55E00"))


saveRDS(dfm_predicted_six6, "figs/data_six6_genotype_year.rds")
saveRDS(sd_six6, "figs/data_six6_sd.rds")
saveRDS(six6_seaage, "figs/data_six6_individual_seaage.rds")




# ---- Predicted mean sea age for each vgll3 genotype in each year  -----------------------------------------------------------------------------------------


newdataF_vgll3 <- as.data.frame(cbind(year_smolt = rep(c(2013:2018), times = 3),
                                  a_vgll3= rep(c(0, 1, 2), each = 6),
                                  d_vgll3 = rep(c(0, 1, 0), each = 6),
                                  a_six6 = rep(mean(df$a_six6), times = 18),   # predict sea age for vgll3 genotypes for an average six6 genotype
                                  d_six6 = rep(mean(df$d_six6), each = 18),
                                  seaAge_correction = rep(0, times = 18)))



predicted <- predict(modF_vgll3_six6_year, newdata = newdataF_vgll3, se.fit = TRUE)
df_predicted_vgll3 <- as_tibble(cbind(predicted = predicted$fit, se = predicted$se.fit,
                                     year_smolt = newdataF_vgll3$year_smolt, 
                                     a_vgll3 = newdataF_vgll3$a_vgll3)) %>% mutate(Sex = "Female")

# Males
newdataM_vgll3 <- as_tibble(cbind(year_smolt = rep(c(2013:2018), times = 3),
                            a_vgll3 = rep(c(0, 1, 2), each = 6),
                            d_vgll3 = rep(c(0, 1, 0),each = 6), 
                            a_six6 = rep(mean(dm$a_six6), times = 18),   
                            d_six6 = rep(mean(dm$d_six6), each = 18),
                            seaAge_correction = rep(0, times = 18)))
predicted <- predict(modM_vgll3_six6_year, newdata = newdataM_vgll3, se.fit = TRUE)

dm_predicted_vgll3 <- as_tibble(cbind(predicted = predicted$fit,
                                     se = predicted$se.fit,
                                     year_smolt = newdataM_vgll3$year_smolt, 
                                     a_vgll3 = newdataM_vgll3$a_vgll3)) %>% mutate(Sex = "Male")

dfm_predicted_vgll3 <- rbind(df_predicted_vgll3, dm_predicted_vgll3)
dfm_predicted_vgll3$Genotype <- with(dfm_predicted_vgll3, ifelse(a_vgll3 == 0, "EE", ifelse(a_vgll3 == 1, "EL", "LL")))





# ------------ Among-year standard deviation of each genotype, corrected for uncertainty ------------------------


dfm_predicted_vgll3 <- dfm_predicted_vgll3 %>% group_by(Sex, Genotype) %>% mutate(variance = var(predicted),
                                                                                meanSE2 = mean(se^2),
                                                                                SD_corrected = sqrt(variance - meanSE2))


# ---------------- Fig 2A  -------------------------------------------------------------------------------------


# Individual sea age data: pool individuals with sea age > 3 with sea age 3 in the plot: 
vgll3_seaage <- d %>% dplyr::select(year_smolt, sex, vgll3, seaAge_firstspawn_123plus)
vgll3_seaage$Genotype <- with(vgll3_seaage, ifelse(vgll3 == 0, "EE", ifelse(vgll3 == 1, "EL", "LL")))
vgll3_seaage$Sex <- vgll3_seaage$sex


# Error-corrected standard deviation for each vgll3 genotype to include in plot:
sd_vgll3 <- dfm_predicted_vgll3 %>% dplyr::select(Sex, Genotype, SD_corrected) %>% slice(1) %>% mutate(SD = round2(SD_corrected))
sd_vgll3$year_smolt <- c(rep(2017, times = 6))
sd_vgll3$predicted <- rep(c(2.9, 3.1, 3.3), times = 2)

# expression with sigma and genotype as subscript: 
sd_vgll3$label[sd_vgll3$Genotype == "EE" & sd_vgll3$Sex == "Female"] <- as.character(expression(sigma[EE] == 0.13))
sd_vgll3$label[sd_vgll3$Genotype == "EL" & sd_vgll3$Sex == "Female"] <- as.character(expression(sigma[EL] == 0.06))
sd_vgll3$label[sd_vgll3$Genotype == "LL" & sd_vgll3$Sex == "Female"] <- as.character(expression(sigma[LL] == 0.13))

sd_vgll3$label[sd_vgll3$Genotype == "EE" & sd_vgll3$Sex == "Male"] <- as.character(expression(sigma[EE] == 0.14))
sd_vgll3$label[sd_vgll3$Genotype == "EL" & sd_vgll3$Sex == "Male"] <- as.character(expression(sigma[EL] == 0.17))
sd_vgll3$label[sd_vgll3$Genotype == "LL" & sd_vgll3$Sex == "Male"] <- as.character(expression(sigma[LL] == 0.25))


dfm_predicted_vgll3 %>% ggplot(aes(x = year_smolt, y = predicted, col = Genotype))+
  geom_line(position = position_dodge(width = 0.3))+
  facet_wrap(.~ Sex)+
  geom_point(position = position_dodge(width = 0.3))+
  geom_text(data = sd_vgll3, aes(label = label), parse = T, show.legend = FALSE)+
  geom_errorbar(aes(ymin = predicted - se, ymax = predicted + se), position = position_dodge(width = 0.3), width = 0.5)+
  geom_jitter(data = vgll3_seaage, aes(x = year_smolt, y = seaAge_firstspawn_123plus, col = Genotype), alpha = 0.1, width = 0.3, height = 0.2, size = 1)+
  scale_y_continuous("Sea age (yr)", breaks = seq(1,3, 1))+
  scale_x_continuous("Year of outmigration", breaks = seq(2013, 2018, 1))+
  scale_color_manual(values = c("#CC79A7", "#009E73","#D55E00"))

saveRDS(dfm_predicted_vgll3, "figs/data_vgll3_genotype_year.rds")
saveRDS(sd_vgll3, "figs/data_vgll3_sd.rds")
saveRDS(vgll3_seaage, "figs/data_vgll3_individual_seaage.rds")


# ---- % of variance among years and six6 genotypes due to genotype-year interaction -----------------------------------------------------------

# In order to calculate how much of the variance among six6 genotypes and years that is due to six6-year interaction, 
# we fit a model that exludes this interaction while keeping the rest constant:  

# excluding six6*year (but controlling for vgll3*year)
modF_six6_year <- lm(seaAge_firstspawn ~  - 1 + offset(seaAge_correction) + factor(year_smolt) + a_six6 + d_six6  + a_vgll3 + d_vgll3 + factor(year_smolt):a_vgll3 + factor(year_smolt):d_vgll3, data = df)
modM_six6_year <- lm(seaAge_firstspawn ~  - 1 + offset(seaAge_correction) + factor(year_smolt) + a_six6 + d_six6  + a_vgll3 + d_vgll3 + factor(year_smolt):a_vgll3 + factor(year_smolt):d_vgll3, data = dm)
summary(modF_six6_year)
# Female

newdataF_six6$d_six6 <- with(newdataF_six6, ifelse(a_six6 == 1, 1, 0))  # when we assume equal a and d effect in each year, we change to correct genotype
predicted <- predict(modF_six6_year, newdata = newdataF_six6, se.fit = TRUE)
df_predicted_six6_without_interaction <- as_tibble(cbind(predicted = predicted$fit,
                                      se = predicted$se.fit,
                                      year_smolt = newdataF_six6$year_smolt, 
                                      a_six6 = newdataF_six6$a_six6)) %>% mutate(Sex = "Female")

predicted <- predict(modM_six6_year, newdata = newdataM_six6, se.fit = TRUE)
dm_predicted_six6_without_interaction <- as_tibble(cbind(predicted = predicted$fit,
                                       se = predicted$se.fit,
                                       year_smolt = newdataM_six6$year_smolt, 
                                       a_six6 = newdataM_six6$a_six6)) %>% mutate(Sex = "Male")
dfm_predicted_six6_without_interaction <- rbind(df_predicted_six6_without_interaction, dm_predicted_six6_without_interaction)


dfm_predicted_six6_without_interaction %>% ggplot(aes(x = year_smolt, y = predicted, col = factor(a_six6)))+
  geom_line()+
  geom_line(data = dfm_predicted_six6, linetype = "dashed")+
  facet_wrap(.~ Sex)+
  ylim(1, 3)

# ---- % of variance among years and vgll3 genotypes due to genotype-year interaction -----------------------------------------------------------

# do the same to calculate the contribution of the vgll3-year interaction 
# excluding vgll3*year (but controlling for six6*year)
modF_vgll3_year <- lm(seaAge_firstspawn ~  - 1 + offset(seaAge_correction) + factor(year_smolt) + a_six6 + d_six6  + a_vgll3 + d_vgll3 + factor(year_smolt):a_six6 + factor(year_smolt):d_six6, data = df)
modM_vgll3_year <- lm(seaAge_firstspawn ~  - 1 + offset(seaAge_correction) + factor(year_smolt) + a_six6 + d_six6  + a_vgll3 + d_vgll3 + factor(year_smolt):a_six6 + factor(year_smolt):d_six6, data = dm)

predicted <- predict(modF_vgll3_year, newdata = newdataF_vgll3, se.fit = TRUE)
df_predicted_vgll3_without_interaction <- as_tibble(cbind(predicted = predicted$fit,
                                                         se = predicted$se.fit,
                                                         year_smolt = newdataF_vgll3$year_smolt, 
                                                         a_vgll3 = newdataF_vgll3$a_vgll3)) %>% mutate(Sex = "Female")

predicted <- predict(modM_vgll3_year, newdata = newdataM_vgll3, se.fit = TRUE)
dm_predicted_vgll3_without_interaction <- as_tibble(cbind(predicted = predicted$fit,
                                                         se = predicted$se.fit,
                                                         year_smolt = newdataM_vgll3$year_smolt, 
                                                         a_vgll3 = newdataM_vgll3$a_vgll3)) %>% mutate(Sex = "Male")
dfm_predicted_vgll3_without_interaction <- rbind(df_predicted_vgll3_without_interaction, dm_predicted_vgll3_without_interaction)



# ---- Table 1: % of variance among years and genotyps explained by genotype-year interaction ---------------------------------------------------------------------------------------------

# How much of the variance among years and genotypes can be attributed to the GxY interaction? 

df_variance_six6_year_interaction <- 
cbind(
  dfm_predicted_six6 %>% group_by(Sex) %>% summarise(total_variance = var(predicted),
                                                     variance_error_corrected = var(predicted)-mean(se^2)),
  dfm_predicted_six6_without_interaction %>% group_by(Sex) %>% summarise(variance_without_interaction = var(predicted))%>% dplyr::select(!Sex)
) %>% 
  mutate("G\u00D7Y" = paste(round(((total_variance - variance_without_interaction)/total_variance)*100), sep = "", "%"),
         "Error" = paste(round(((total_variance - variance_error_corrected)/total_variance)*100), sep = "", "%"),
         "SD" = sqrt(variance_error_corrected)) 


df_variance_vgll3_year_interaction <- 
  cbind(
    dfm_predicted_vgll3 %>% group_by(Sex) %>% summarise(total_variance = var(predicted),
                                                       variance_error_corrected = var(predicted)-mean(se^2)),
    dfm_predicted_vgll3_without_interaction %>% group_by(Sex) %>% summarise(variance_without_interaction = var(predicted))%>% dplyr::select(!Sex)
  ) %>% 
  mutate("G\u00D7Y" = paste(round(((total_variance - variance_without_interaction)/total_variance)*100), sep = "", "%"),
         "Error" = paste(round(((total_variance - variance_error_corrected)/total_variance)*100), sep = "", "%"),
         "SD" = sqrt(variance_error_corrected)) 


df_variance_genotype_year_interaction <- 
  rbind(
    df_variance_vgll3_year_interaction %>% mutate(Gene = c("vgll3", NA)), 
    df_variance_six6_year_interaction %>% mutate(Gene = c("six6", NA))
  ) %>%  
  dplyr::select(!variance_without_interaction & !variance_error_corrected) %>% 
  relocate(Gene, everything()) 


style_table(df_variance_genotype_year_interaction)

