
# Evolvability 

# Average single-locus evolvability of vgll3 and six6
# Total evolvability in sea age given h2 = 0.5 and h2 = 1
# Contribution of vgll3 + six6 to total evolvability

# single locus evolvability in each smolt year 
# error-corrected among-year standard deviation in single-locus evolvabilities 

# Figs 2C and 3C 


source("06-add-dom-effects-by-year.R")

# ---- Additive genetic variance formula ----------------------------------

add_var_calc <- function(p, a, d){                               # Additive genetic variance formula
  2*p*(1-p)*(a + d*(1-2*p))^2                                    # p: allele frequency, a is additive effect and d is dominance effect
}

bias_calc <- function(p, var_a, var_d, cov_ad){                  # Bias formula 
  2*p*(1-p)*(var_a + (1-2*p)^2*var_d + 2*(1-2*p)*cov_ad)         # p: allele frequency  
}                                                                # var_a: error variance of additive effect
                                                                 # var_d: error variance of dominance effect 
                                                                 # error covariance of additive and dominance effect 


# ---- L allele frequencies of vgll3 and six6 over all individuals --------

Lfreq_vgll3 <- d %>% 
  group_by(vgll3) %>%
  summarise(n = n()) %>%                                                       # number of individuals per genotype
  summarise(p = (sum(n[vgll3 == 2])*2 + sum(n[vgll3 == 1]))/(sum(n)*2)) %>%    # Frequency of L allele is (2*n LL fish + 1*n EL fish) / total 2*total number of fish
  pull()

Lfreq_six6 <- d %>% 
  group_by(six6) %>%
  summarise(n = n()) %>%                                                    
  summarise(p = (sum(n[six6 == 2])*2 + sum(n[six6 == 1]))/(sum(n)*2)) %>%    
  pull()

# ---- Mean sea age of females and males ------------------------------------

mean_seaage_F <- mean(df$seaAge_firstspawn) 
mean_seaage_M <- mean(dm$seaAge_firstspawn)


# ---- Average single-locus evolvability of vgll3 and six6 ------------------------------------

# Females vgll3
a <- coef(modF_vgll3_six6)["a_vgll3"]                          # average additive effect of vgll3 in females 
d <- coef(modF_vgll3_six6)["d_vgll3"]                          # average dominance effect of vgll3 in females 

var_a <- vcov(modF_vgll3_six6)["a_vgll3", "a_vgll3"]           # error variance of additive effect
var_d <- vcov(modF_vgll3_six6)["d_vgll3", "d_vgll3"]           # error variance of dominance effect
cov_ad <- vcov(modF_vgll3_six6)["a_vgll3", "d_vgll3"]          # error covariance of additive and dominance effect 

va <- add_var_calc(Lfreq_vgll3, a, d)                          # calculate raw additive genetic variance of vgll3 
bias <- bias_calc(Lfreq_vgll3, var_a, var_d, cov_ad)           # calculate bias
bias

va_corrected_vgll3_F <- va - bias                              # bias-corrected additive genetic variance 

evolvability_vgll3_F <- va_corrected_vgll3_F/mean_seaage_F^2   # calculate single-locus evolvability of vgll3  
evolvability_vgll3_F*100                                       # single-locus evolvability of vgll3 in %

# Males vgll3

a <- coef(modM_vgll3_six6)["a_vgll3"]
d <- coef(modM_vgll3_six6)["d_vgll3"]

var_a <- vcov(modM_vgll3_six6)["a_vgll3", "a_vgll3"]
var_d <- vcov(modM_vgll3_six6)["d_vgll3", "d_vgll3"]
cov_ad <- vcov(modM_vgll3_six6)["a_vgll3", "d_vgll3"]

va <- add_var_calc(Lfreq_vgll3, a, d)
bias <- bias_calc(Lfreq_vgll3, var_a, var_d, cov_ad)
bias

va_corrected_vgll3_M <- va - bias

evolvability_vgll3_M <- va_corrected_vgll3_M/mean_seaage_M^2  
evolvability_vgll3_M*100



# Females six6

a <- coef(modF_vgll3_six6)["a_six6"]
d <- coef(modF_vgll3_six6)["d_six6"]

var_a <- vcov(modF_vgll3_six6)["a_six6", "a_six6"]
var_d <- vcov(modF_vgll3_six6)["d_six6", "d_six6"]
cov_ad <- vcov(modF_vgll3_six6)["a_six6", "d_six6"]

va <- add_var_calc(Lfreq_six6, a, d)
bias <- bias_calc(Lfreq_six6, var_a, var_d, cov_ad)
bias

va_corrected_six6_F <- va - bias

evolvability_six6_F <- va_corrected_six6_F/mean_seaage_F^2  
evolvability_six6_F*100


# Males six6

a <- coef(modM_vgll3_six6)["a_six6"]
d <- coef(modM_vgll3_six6)["d_six6"]


var_a <- vcov(modM_vgll3_six6)["a_six6", "a_six6"]
var_d <- vcov(modM_vgll3_six6)["d_six6", "d_six6"]
cov_ad <- vcov(modM_vgll3_six6)["a_six6", "d_six6"]


va <- add_var_calc(Lfreq_six6, a, d)
bias <- bias_calc(Lfreq_six6, var_a, var_d, cov_ad)
bias

va_corrected_six6_M <- va - bias

evolvability_six6_M <- va_corrected_six6_M/mean_seaage_M^2  
evolvability_six6_M*100


# --- Total evolvability assuming h2 = 0.5 ------------------------------------

# females
vp_F <- var(df$seaAge_firstspawn)                       # total phenotypic variance in sea age over all females
vp_F
(va_corrected_vgll3_F + va_corrected_six6_F)/vp_F*100   # 9.6% of the total phenotypic variance explained by additive genetic effects of vgll3 and six6 

va_F <- vp_F*0.5                                        # total additive genetic variance asuming h2 = 0.5

evolvability_tot_F <- va_F/mean_seaage_F^2              
100*evolvability_tot_F                                                                      # total evolvability of 3.28 in females 

(evolvability_vgll3_F/evolvability_tot_F*100)+(evolvability_six6_F/evolvability_tot_F*100)  # 19% of total evolvability due to vgll3 + six6 

# Males 
vp_M <- var(dm$seaAge_firstspawn)
vp_M

(va_corrected_vgll3_M + va_corrected_six6_M)/vp_M*100 # 7.4%  of total phenotypic variance explained by vgll3 and six6additive genetic var

va_M <- vp_M*0.5
evolvability_tot_M <- va_M/mean_seaage_M^2
100*evolvability_tot_M                                                                        # total evolvability of 8.11% in males 
(evolvability_vgll3_M/evolvability_tot_M*100)  + (evolvability_six6_M/evolvability_tot_M*100) # 14.8% of total due to vgll3 + six6


# Assuming that all variation in sea age is due to additive genetic variation (h2 = 1): 
evolvability_tot_upper_M <- vp_M/mean_seaage_M^2
evolvability_vgll3_M/evolvability_tot_upper_M*100+ evolvability_six6_M/evolvability_tot_upper_M*100

evolvability_tot_upper_F <- vp_F/mean_seaage_F^2
evolvability_vgll3_F/evolvability_tot_upper_F*100+ evolvability_six6_F/evolvability_tot_upper_F*100


# --- Single-locus evolvabilities per smolt year --------------------------------------------

# ---- vgll3  ----------------------------------------------------

# additive and dominance effect per year 

d_genetic_variance <- d_add_dom %>%                               # table created in 06-add-dom-effects-by-year.R)
  filter(Gene == "vgll3") %>%
  dplyr::select(Sex, Gene, year_smolt, Term, estimate) %>% 
  pivot_wider(names_from = "Term", values_from = c("estimate")) 


d_genetic_variance$var_a <- c(vcov(modF_vgll3_six6_year13)["a_vgll3", "a_vgll3"],     # error variance of additive effect in each year
                              vcov(modF_vgll3_six6_year14)["a_vgll3", "a_vgll3"],
                              vcov(modF_vgll3_six6_year15)["a_vgll3", "a_vgll3"],
                              vcov(modF_vgll3_six6_year16)["a_vgll3", "a_vgll3"],
                              vcov(modF_vgll3_six6_year17)["a_vgll3", "a_vgll3"],
                              vcov(modF_vgll3_six6_year18)["a_vgll3", "a_vgll3"],
                              vcov(modM_vgll3_six6_year13)["a_vgll3", "a_vgll3"],
                              vcov(modM_vgll3_six6_year14)["a_vgll3", "a_vgll3"],
                              vcov(modM_vgll3_six6_year15)["a_vgll3", "a_vgll3"],
                              vcov(modM_vgll3_six6_year16)["a_vgll3", "a_vgll3"],
                              vcov(modM_vgll3_six6_year17)["a_vgll3", "a_vgll3"],
                              vcov(modM_vgll3_six6_year18)["a_vgll3", "a_vgll3"])

d_genetic_variance$var_d <- c(vcov(modF_vgll3_six6_year13)["d_vgll3", "d_vgll3"],     # error variance of dominance effect in each year
                              vcov(modF_vgll3_six6_year14)["d_vgll3", "d_vgll3"],
                              vcov(modF_vgll3_six6_year15)["d_vgll3", "d_vgll3"],
                              vcov(modF_vgll3_six6_year16)["d_vgll3", "d_vgll3"],
                              vcov(modF_vgll3_six6_year17)["d_vgll3", "d_vgll3"],
                              vcov(modF_vgll3_six6_year18)["d_vgll3", "d_vgll3"],
                              vcov(modM_vgll3_six6_year13)["d_vgll3", "d_vgll3"],
                              vcov(modM_vgll3_six6_year14)["d_vgll3", "d_vgll3"],
                              vcov(modM_vgll3_six6_year15)["d_vgll3", "d_vgll3"],
                              vcov(modM_vgll3_six6_year16)["d_vgll3", "d_vgll3"],
                              vcov(modM_vgll3_six6_year17)["d_vgll3", "d_vgll3"],
                              vcov(modM_vgll3_six6_year18)["d_vgll3", "d_vgll3"])

d_genetic_variance$cov_ad <- c(vcov(modF_vgll3_six6_year13)["a_vgll3", "d_vgll3"],    # error-covariance between additive and dominance effect in each year
                               vcov(modF_vgll3_six6_year14)["a_vgll3", "d_vgll3"],
                               vcov(modF_vgll3_six6_year15)["a_vgll3", "d_vgll3"],
                               vcov(modF_vgll3_six6_year16)["a_vgll3", "d_vgll3"],
                               vcov(modF_vgll3_six6_year17)["a_vgll3", "d_vgll3"],
                               vcov(modF_vgll3_six6_year18)["a_vgll3", "d_vgll3"],
                               vcov(modM_vgll3_six6_year13)["a_vgll3", "d_vgll3"],
                               vcov(modM_vgll3_six6_year14)["a_vgll3", "d_vgll3"],
                               vcov(modM_vgll3_six6_year15)["a_vgll3", "d_vgll3"],
                               vcov(modM_vgll3_six6_year16)["a_vgll3", "d_vgll3"],
                               vcov(modM_vgll3_six6_year17)["a_vgll3", "d_vgll3"],
                               vcov(modM_vgll3_six6_year18)["a_vgll3", "d_vgll3"])


# mean sea age per year for each sex from model that controls for sampling variation 
d_mean_seaage <- rbind(
  tidy(modF) %>% mutate(year_smolt = min(df$year_smolt):max(df$year_smolt),
                        Sex = "Female",
                        mean_seaage = estimate),
  tidy(modM) %>% mutate(year_smolt = min(dm$year_smolt):max(dm$year_smolt),
                        Sex = "Male",
                        mean_seaage = estimate))

d_evolvability_vgll3 <-
  d_genetic_variance %>%
  left_join(d_mean_seaage %>% dplyr::select(Sex, year_smolt, mean_seaage) , by = c("Sex", "year_smolt")) %>%
  mutate(Lfreq = Lfreq_vgll3,
         va = add_var_calc(Lfreq, additive, dominance),
         bias = bias_calc(Lfreq, var_a, var_d, cov_ad),
         va_bias_corr = ifelse(va > bias, va - bias, 0),
         evolvability = ifelse(va_bias_corr > 0, va_bias_corr/(mean_seaage^2), 0),
         Locus = "vgll3")


style_table(d_evolvability_vgll3) %>% colformat_num(j = 3, big.mark = "")


# ---- six6 ----------------------------------------------------

# additive and dominance effect per year 

d_genetic_variance <- d_add_dom %>% 
  filter(Gene == "six6") %>%
  dplyr::select(Sex, Gene, year_smolt, Term, estimate) %>% 
  pivot_wider(names_from = "Term", values_from = c("estimate")) 


################
d_genetic_variance$var_a <- c(vcov(modF_vgll3_six6_year13)["a_six6", "a_six6"],             # error variances additive effect
                              vcov(modF_vgll3_six6_year14)["a_six6", "a_six6"],
                          #  vcov(modF_vgll3_six6_year15)["a_six6", "a_six6"],              # exclude 2015 in females because one missing genotype
                              vcov(modF_vgll3_six6_year16)["a_six6", "a_six6"],
                              vcov(modF_vgll3_six6_year17)["a_six6", "a_six6"],
                              vcov(modF_vgll3_six6_year18)["a_six6", "a_six6"],
                              vcov(modM_vgll3_six6_year13)["a_six6", "a_six6"],
                              vcov(modM_vgll3_six6_year14)["a_six6", "a_six6"],
                              vcov(modM_vgll3_six6_year15)["a_six6", "a_six6"],
                              vcov(modM_vgll3_six6_year16)["a_six6", "a_six6"],
                              vcov(modM_vgll3_six6_year17)["a_six6", "a_six6"],
                              vcov(modM_vgll3_six6_year18)["a_six6", "a_six6"])

d_genetic_variance$var_d <- c(vcov(modF_vgll3_six6_year13)["d_six6", "d_six6"],             # error variances dominance effect
                              vcov(modF_vgll3_six6_year14)["d_six6", "d_six6"],
                   #          vcov(modF_vgll3_six6_year15)["d_six6", "d_six6"],
                              vcov(modF_vgll3_six6_year16)["d_six6", "d_six6"],
                              vcov(modF_vgll3_six6_year17)["d_six6", "d_six6"],
                              vcov(modF_vgll3_six6_year18)["d_six6", "d_six6"],
                              vcov(modM_vgll3_six6_year13)["d_six6", "d_six6"],
                              vcov(modM_vgll3_six6_year14)["d_six6", "d_six6"],
                              vcov(modM_vgll3_six6_year15)["d_six6", "d_six6"],
                              vcov(modM_vgll3_six6_year16)["d_six6", "d_six6"],
                              vcov(modM_vgll3_six6_year17)["d_six6", "d_six6"],
                              vcov(modM_vgll3_six6_year18)["d_six6", "d_six6"])



d_genetic_variance$cov_ad <- c(vcov(modF_vgll3_six6_year13)["a_six6", "d_six6"],             # error-covariances between additive and dominance effect
                               vcov(modF_vgll3_six6_year14)["a_six6", "d_six6"],
                  #            vcov(modF_vgll3_six6_year15)["a_six6", "d_six6"],
                               vcov(modF_vgll3_six6_year16)["a_six6", "d_six6"],
                               vcov(modF_vgll3_six6_year17)["a_six6", "d_six6"],
                               vcov(modF_vgll3_six6_year18)["a_six6", "d_six6"],
                               vcov(modM_vgll3_six6_year13)["a_six6", "d_six6"],
                               vcov(modM_vgll3_six6_year14)["a_six6", "d_six6"],
                               vcov(modM_vgll3_six6_year15)["a_six6", "d_six6"],
                               vcov(modM_vgll3_six6_year16)["a_six6", "d_six6"],
                               vcov(modM_vgll3_six6_year17)["a_six6", "d_six6"],
                               vcov(modM_vgll3_six6_year18)["a_six6", "d_six6"])

d_evolvability_six6 <- 
  d_genetic_variance %>%
  left_join(d_mean_seaage %>% dplyr::select(Sex, year_smolt, mean_seaage) , by = c("Sex", "year_smolt")) %>% 
  mutate(Lfreq = Lfreq_six6,
         va = add_var_calc(Lfreq, additive, dominance),
         bias = bias_calc(Lfreq, var_a, var_d, cov_ad),
         va_bias_corr = ifelse(va > bias, va - bias, 0),
         evolvability = ifelse(va_bias_corr > 0, va_bias_corr/(mean_seaage^2), 0),
         Locus = "six6")

style_table(d_evolvability_six6) %>% colformat_num(j = 3, big.mark = "")


# range in evolvabilities 
with(subset(d_evolvability_vgll3, Sex == "Female"), range(evolvability*100))
with(subset(d_evolvability_vgll3, Sex == "Male"), range(evolvability*100))

with(subset(d_evolvability_six6, Sex == "Female"), range(evolvability*100))
with(subset(d_evolvability_six6, Sex == "Male"), range(evolvability*100))


# ---- Error-corrected among-year standard deviations in evolvability --------------------------------


# Females vgll3 
raw_var <- with(subset(d_evolvability_vgll3, Sex == "Female"), var(cbind(additive, dominance))) # raw variance in a and d among years
err_var <- with(subset(d_evolvability_vgll3, Sex == "Female"), 
                matrix(apply(cbind(var_a, cov_ad, cov_ad, var_d), 2, mean), ncol = 2))          # mean error variances and error covariances
corrected_var <- raw_var - err_var                                                              # error-corrected among-year variance
corrected_var[1:3] <- 0                                                                         # No variance left in the a parameter.

d_corrected_var_F_vgll3 <- corrected_var

# sample from 1000 values of a and d from their error-corrected among-year variance matrix:
ad_samples <- mvrnorm(n = 1000, mu = c(coef(modF_vgll3_six6)["a_vgll3"],  coef(modF_vgll3_six6)["d_vgll3"]), Sigma = corrected_var)

e_vgll3_female <- 100*add_var_calc(ad_samples[,1], ad_samples[, 2], Lfreq_vgll3)/mean_seaage_F^2
hist(e_vgll3_female)
sd(e_vgll3_female) # The error-corrected standard deviation in evolvability is 0.37%

# Male vgll3 
raw_var <- with(subset(d_evolvability_vgll3, Sex == "Male"), var(cbind(additive, dominance)))
err_var <- with(subset(d_evolvability_vgll3, Sex == "Male"), 
                matrix(apply(cbind(var_a, cov_ad, cov_ad, var_d), 2, mean), ncol = 2))
corrected_var1 <- raw_var - err_var
corrected_correlations <- cov2cor(corrected_var1)
corrected_correlations[2:3]<- -1 # correlations cannot be lower than -1
corrected_var <- corrected_correlations*sqrt(diag(corrected_var1))%*%t(sqrt(diag(corrected_var1)))

d_corrected_var_M_vgll3 <- corrected_var

ad_samples <- mvrnorm(n = 1000, mu = c(coef(modM_vgll3_six6)["a_vgll3"], coef(modM_vgll3_six6)["d_vgll3"]), Sigma = corrected_var)

e_vgll3_male <- 100*add_var_calc(ad_samples[,1], ad_samples[, 2], p = Lfreq_vgll3)/mean_seaage_M^2
hist(e_vgll3_male)
sd(e_vgll3_male)  # The standard deviation is 0.30%


# Female six6
raw_var <- with(subset(d_evolvability_six6, Sex == "Female"), var(cbind(additive, dominance)))
err_var <- with(subset(d_evolvability_six6, Sex == "Female"), 
                matrix(apply(cbind(var_a, cov_ad, cov_ad, var_d), 2, mean), ncol = 2))
corrected_var1 <- raw_var - err_var
corrected_correlations <- cov2cor(corrected_var1)
corrected_correlations[2:3]<- 1 # correlations cannot be lower than -1
corrected_var <- corrected_correlations*sqrt(diag(corrected_var1))%*%t(sqrt(diag(corrected_var1)))

d_corrected_var_F_six6 <- corrected_var



ad_samples <- mvrnorm(n = 1000, mu = c(coef(modM_vgll3_six6)["a_six6"], coef(modM_vgll3_six6)["d_six6"]), Sigma = corrected_var)

e_six6_female <- 100*add_var_calc(ad_samples[,1], ad_samples[, 2], p = Lfreq_six6)/mean_seaage_F^2
hist(e_six6_female)
sd(e_six6_female) # The standard deviation is 0.09%


## ---- average single-locus evolvability of vgll3 in males +- standard deviation:  

100*evolvability_vgll3_M - sd(e_vgll3_male)
100*evolvability_vgll3_M + sd(e_vgll3_male)


with(subset(d_evolvability_vgll3, Sex == "Female"), range(evolvability*100))
with(subset(d_evolvability_vgll3, Sex == "Male"), range(evolvability*100))

with(subset(d_evolvability_six6, Sex == "Female"), range(evolvability*100))
with(subset(d_evolvability_six6, Sex == "Male"), range(evolvability*100))

# ---- Figs 2C and 3C single-locus evolvability each year  --------------------------------------------

# Labels with error-corrected among-year standard deviations in evolvability  
sd_vgll3_evolv <- 
as.data.frame(cbind(Sex = c("Female", "Male"),
      label = c(as.character(expression(sigma[e] == "0.38")), as.character(expression(sigma[e] == "0.32"))))) 

sd_vgll3_evolv$evolvability <- c(0.025)
sd_vgll3_evolv$year_smolt <- c(2014.5)
saveRDS(sd_vgll3_evolv, "figs/sd_vgll3_evolv.rds")


d_evolvability_vgll3 %>% 
  ggplot(aes(x = year_smolt, y = (evolvability*100)))+
  geom_col()+
  facet_grid(.~ Sex)+
  geom_text(data = sd_vgll3_evolv, aes(label = label), parse = T)+
  scale_y_continuous(breaks = seq(0, 3, 0.5))+
  scale_x_continuous("Year of outmigration", breaks = seq(2013, 2018, 1))+
  ylab("Evolvability (%)")


sd_six6_evolv <- 
  as.data.frame(cbind(Sex = c("Female", "Male"),
                      label = c(as.character(expression(sigma[e] == "0.09")), as.character(expression(sigma[e] == "0.00"))))) 

sd_six6_evolv$evolvability <- c(0.0068)
sd_six6_evolv$year_smolt <- c(2014.5)
saveRDS(sd_six6_evolv, "figs/sd_six6_evolv.rds")


d_evolvability_six6 %>% 
  ggplot(aes(x = year_smolt, y = (evolvability*100)))+
  geom_col()+
  geom_text(data = sd_six6_evolv, aes(label = label), parse = T)+
  facet_grid(.~ Sex)+
  scale_x_continuous("Year of outmigration", breaks = seq(2013, 2018, 1))+
  ylab("Evolvability (%)")


saveRDS(d_evolvability_six6, "d_evolvability_six6.rds")
saveRDS(d_evolvability_vgll3, "d_evolvability_vgll3.rds")
