
source("01-libs-functions.R")

# --- Load dataset with individual sea age and genotype -------

# This dataset includes all wild individual salmon with information on sea age (N = 2534)
d_scale <- read.table("data/data-raunsgard-etal-2023-MEC-individual-seaage-genotype.csv",sep=",", header = TRUE)

# ----  Load return data ---------------------------------------
d_returns <- read.table("data/data-raunsgard-etal-2023-MEC-yearly-returns.csv", sep = ",",  header = TRUE)


# ---- Correcting for sampling variation -----------------------

# We use the sea age distribution in each capture year from the scale dataset, in combination with the total number of returning salmon
# and proportion hatchery produced salmon in each return year, to calculate mean sea age in each smolt cohort, corrected for variation
# in sampling/returns in each capture year 


##### ----- Calculate the "true" mean sea age -------####

# Data used to calculate the "true" number of returning salmon in each age group
# Assuming that the sea age distributions in the scale data set represents the true sea age distribution in a given capture year, we can calculate the total number of fish in each age age group in the capture year by combining the proportion of each sea age in the scale data with the number of returning fish that year.

table_data_for_seaage_correction <- style_table(
  d_scale %>% group_by(year_capture, seaAge) %>% 
    summarise(n = n()) %>% ungroup() %>% 
    pivot_wider(values_from = n, names_from = seaAge, names_prefix = "seaage") %>%  
    left_join(d_returns %>% dplyr::select(year_capture, N_returns, prop_hatchery) %>% 
                mutate(prop_hatchery = round(prop_hatchery, 3), N_returns = round(N_returns)), by = "year_capture")
) %>% 
  colformat_num(big.mark = "")
table_data_for_seaage_correction


# N fish per sea age in capture year
# calculate number of fish and proportions of each sea age in each capture year
N_capture <- d_scale %>% 
  group_by(year_capture, seaAge, seaAge_firstspawn) %>% 
  summarise(n_obs = n()) %>%                                      # observed number of each sea age in each capture year  
  ungroup() %>%  
  group_by(year_capture) %>% 
  mutate(n_obs_tot = sum(n_obs)) %>%                              # observed number total 
  ungroup() 
N_capture <- N_capture %>% left_join(d_returns %>% dplyr::select(year_capture, N_returns_adj), by = "year_capture")  # add total number of returned fish (simulated by VRL from catch and exploitation rate)
N_capture$N_true <- with(N_capture, n_obs/n_obs_tot*N_returns_adj)  # true total number of fish per sea age in each capture year


# N fish per sea age in smolt year
# We use the "true" number of fish in each sea age in a given capture year and assign them to their smolt cohorts
# to get the total number of fish in each smolt cohort, 
# and  calculate the "true" smolt-cohort mean sea age at FIRST REPRODUCTION (important to get it right for repeat spawners):


N_smolt <- N_capture %>% 
  dplyr::select(year_capture, seaAge, seaAge_firstspawn, N_true) %>% 
  mutate(year_smolt = year_capture - seaAge) %>%                                     # smolt year variable
  group_by(year_smolt) %>% 
  mutate(Ntot_true_smolt = sum(N_true),                                              # total number of fish in smolt cohort true (including repeat spawners)
         mean_seaage_true_smolt = sum(N_true*seaAge_firstspawn)/Ntot_true_smolt) %>% # smolt-cohort mean sea age at FIRST REPRODUCTION, corrected for sampling
  ungroup()  
N_smolt <- subset(N_smolt, year_smolt > 2012 & year_smolt < 2019)                    # Only include smolt cohorts that had sampling at least 1, 2 and 3 years after smolt out-migration
N_smolt <- N_smolt %>% group_by(year_smolt) %>% dplyr::select(year_smolt, mean_seaage_true_smolt) %>% slice(1)


# ---- Datasets used in the analyses ----

# All individuals must have information on:
# sex
# seaAge_firstspawn
# vgll3
# six6
# only include smolt cohorts with at least three subsequent years of sampling to ensure that it has been possible to sample 1SW, 2SW and 3SW fish 

d <- d_scale %>% filter(!is.na(sex) & !is.na(seaAge_firstspawn) & !is.na(vgll3) & !is.na(six6) & year_smolt > 2012 & year_smolt < 2019 ) %>% 
  left_join(N_smolt %>% dplyr::select(mean_seaage_true_smolt, year_smolt), by = "year_smolt") %>%    # add smolt-cohort mean sea age corrected for sampling  
  group_by(year_smolt) %>% 
  mutate(mean_seaage_observed_smolt = mean(seaAge_firstspawn),                                # calculate raw mean smolt cohort mean sea age 
         seaAge_correction = mean_seaage_observed_smolt - mean_seaage_true_smolt) %>%         # offset variable
  ungroup()

df <- subset(d, sex == "Female")
dm <- subset(d, sex == "Male")


df$d_six6 <- with(df, ifelse(year_smolt == 2015, 0, d_six6)) # cannot estimate dominance when we only have the EL & LL genotype, therefore set d_six6 to 0 for all females in 2015



